from flask import abort, flash, make_response, render_template, url_for
from flask_login import current_user, login_required
from werkzeug.utils import secure_filename
from . import services
from .. import db
from ..jobs.forms import AddFileSetupJobForm, AddNLPJobForm, AddOCRJobForm
from ..models import Job, JobInput
import json
import logging
import os


SERVICES = {'corpus_analysis': {'name': 'Corpus analysis'},
            'file-setup': {'name': 'File setup',
                           'resources': {'mem_mb': 4096, 'n_cores': 4},
                           'form': AddFileSetupJobForm},
            'nlp': {'name': 'Natural Language Processing',
                    'resources': {'mem_mb': 4096, 'n_cores': 2},
                    'form': AddNLPJobForm},
            'ocr': {'name': 'Optical Character Recognition',
                    'resources': {'mem_mb': 8192, 'n_cores': 4},
                    'form': AddOCRJobForm}}


@services.route('/<service>', methods=['GET', 'POST'])
@login_required
def service(service):
    if service not in SERVICES:
        abort(404)
    if service == 'corpus_analysis':
        return render_template('services/{}.html.j2'.format(service),
                               title=SERVICES[service]['name'])
    form = SERVICES[service]['form'](prefix='add-job-form')
    if form.is_submitted():
        if not form.validate():
            return make_response(form.errors, 400)
        service_args = []
        if service == 'nlp':
            service_args.append('-l {}'.format(form.language.data))
            if form.check_encoding.data:
                service_args.append('--check-encoding')
        if service == 'ocr':
            service_args.append('-l {}'.format(form.language.data))
            if form.binarization.data:
                service_args.append('--binarize')
        job = Job(creator=current_user,
                  description=form.description.data,
                  mem_mb=SERVICES[service]['resources']['mem_mb'],
                  n_cores=SERVICES[service]['resources']['n_cores'],
                  service=service, service_args=json.dumps(service_args),
                  service_version=form.version.data,
                  status='preparing', title=form.title.data)
        db.session.add(job)
        db.session.commit()
        try:
            os.makedirs(job.path)
        except OSError:
            logging.error('Make dir {} led to an OSError!'.format(job.path))
            db.session.delete(job)
            db.session.commit()
            flash('Internal Server Error', 'error')
            return make_response(
                {'redirect_url': url_for('.service', service=service)}, 500)
        else:
            for file in form.files.data:
                filename = secure_filename(file.filename)
                job_input = JobInput(filename=filename, job=job)
                file.save(job_input.path)
                db.session.add(job_input)
            job.status = 'submitted'
            db.session.commit()
            flash('Job "{}" added'.format(job.title), 'job')
            return make_response(
                {'redirect_url': url_for('jobs.job', job_id=job.id)}, 201)
    return render_template('services/{}.html.j2'.format(service),
                           form=form, title=SERVICES[service]['name'])
